#include "mbed.h"
#include "rtos.h"
#include "Servo.h"
#include "C12832.h"
Servo s1(p21);
Servo s2(p22);

AnalogIn p1(p19); // Sonar sensor 1 17 for the sensor or 19 for the potential meter.
//AnalogIn p2(p20); // Sonar sensor 2 (not used yet)
Mutex inData_mutex;
Mutex outData_mutex;

// Globel variables
float inData;
float outData;
C12832 lcd(p5, p7, p6, p8, p11);// lcd is an object from class c12832 initialised by p5,p7....

/* Thread Sonar - handles the input data from the sonar sensor, and display on the LCD screen.
    @update inData */
void sonar_thread(void const *args) {
    while (true) {
        inData_mutex.lock();
        inData = p1.read();
        inData_mutex.unlock();
        Thread::wait(25);
    }
}

/* Thread Control - 
    @update outData */
void control_thread(void const *args) {
    while (true) {
        inData_mutex.lock();
        outData_mutex.lock();
        // The servo works from 0 to 1 volt and in approx 110 degree
        float voltage = inData; // local variable to play with
        // Logic code comes here
        
        outData = voltage;  // Update output_data global variable
        // Display values on the LCD screen
        lcd.cls();      // clear the display
        lcd.locate(0,3);// the location where you want your charater to be displayed
        lcd.printf("In: %f, Out: %f", inData, outData);
        // Handles the thread
        inData_mutex.unlock();
        outData_mutex.unlock();
        Thread::wait(20);
    }
}

/* Thread Servo - handles the output data from the control thread, and pass to the servo.
    @update s1, s2 */
void servo_thread(void const *args) {
    while (true) {
        outData_mutex.lock();
        s1 = outData;
        s2 = outData;
        outData_mutex.unlock();
        Thread::wait(25);
    }
}

/* Main method - Start threads and delay them by waiting. */
int main()
{
    Thread thread_1(sonar_thread); // Start Sonar Thread
    Thread thread_2(control_thread); // Start Control Thread
    Thread thread_3(servo_thread); // Start Servo Thread
    // Delay the threads
    while (true) {
        Thread::wait(10);
    }
}
